function[bp, cbp, bps, cbps, bpx, cbpx, bpy, cbpy]=create_filterset(s1,s2,Ecut)
% [bp cbp]=create_DOP(s1,s2,Ecut)
% creates a DOP (difference of Poisson kernels) bandpass filter BP and
% its Riesz transform CBP (complex), based on the parameters
% S1: finer scale of DOP (at least: 1)
% S2: coarser scale of DOP (must be larger than S1)
% Ecut: maximal cut-off energy (in percentage)
 
% Copyright (c) 2003 Michael Felsberg
% $Revision: 1.1 $  $Date: 2003/10/02 09:55:12 $
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program; if not, write to the Free Software
%    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

if nargin<3
  error('Scales and cut-off energy must be specified!');
end

E=(s1^-2+s2^-2-8/(s1+s2)^2)/8/pi;
Eapp=(1-Ecut/100)*E;

S=s2;
[x,y]=meshgrid(-S:S,-S:S);
tp1=s1/(2*pi)./(s1^2+x.^2+y.^2).^(3/2);
tp2=s2/(2*pi)./(s2^2+x.^2+y.^2).^(3/2);
bp=tp1-tp2;

while sum(bp(:).^2)<Eapp
  S=S+1;
  [x,y]=meshgrid(-S:S,-S:S);
  tp1=s1/(2*pi)./(s1^2+x.^2+y.^2).^(3/2);
  tp2=s2/(2*pi)./(s2^2+x.^2+y.^2).^(3/2);
  bp=tp1-tp2;
end

cdc=sum(tp1(:))/sum(tp2(:));
bp=tp1-cdc*tp2; % ensure zero DC

c11=x./(2*pi*(s1^2+x.^2+y.^2).^(3/2));
c12=x./(2*pi*(s2^2+x.^2+y.^2).^(3/2));
c21=y./(2*pi*(s1^2+x.^2+y.^2).^(3/2));
c22=y./(2*pi*(s2^2+x.^2+y.^2).^(3/2));
cbp=c11-cdc*c12+i*(c21-cdc*c22);

bp=sqrt(E/sum(bp(:).^2))*bp; % ensure correct energy
cbp=sqrt(E/sum(real(cbp(:).*conj(cbp(:)))))*cbp; % ensure correct energy

E=3*(s1^-4+s2^-4-32/(s1+s2)^4)/16/pi;

tp1=(-2*s1^2+x.^2+y.^2)./(2*pi)./(s1^2+x.^2+y.^2).^(5/2);
tp2=(-2*s2^2+x.^2+y.^2)./(2*pi)./(s2^2+x.^2+y.^2).^(5/2);
bps=tp1-tp2;

c11=-3*s1*x./(2*pi*(s1^2+x.^2+y.^2).^(5/2));
c12=-3*s2*x./(2*pi*(s2^2+x.^2+y.^2).^(5/2));
c21=-3*s1*y./(2*pi*(s1^2+x.^2+y.^2).^(5/2));
c22=-3*s2*y./(2*pi*(s2^2+x.^2+y.^2).^(5/2));
cbps=c11-c12+i*(c21-c22);

bps=sqrt(E/sum(bps(:).^2))*bps; % ensure correct energy
cbps=sqrt(E/sum(real(cbps(:).*conj(cbps(:)))))*cbps; % ensure correct energy

bpx=real(cbps);

c11=(s1^2-2*x.^2+y.^2)./(2*pi*(s1^2+x.^2+y.^2).^(5/2));
c12=(s2^2-2*x.^2+y.^2)./(2*pi*(s2^2+x.^2+y.^2).^(5/2));
c21=-3*x.*y./(2*pi*(s1^2+x.^2+y.^2).^(5/2));
c22=-3*x.*y./(2*pi*(s2^2+x.^2+y.^2).^(5/2));
cbpx=c11-c12+i*(c21-c22);
cbpx=sqrt(E/2/sum(real(cbpx(:).*conj(cbpx(:)))))*cbpx; % ensure correct energy

bpy=imag(cbps);

c21=(s1^2+x.^2-2*y.^2)./(2*pi*(s1^2+x.^2+y.^2).^(5/2));
c22=(s2^2+x.^2-2*y.^2)./(2*pi*(s2^2+x.^2+y.^2).^(5/2));
c11=-3*x.*y./(2*pi*(s1^2+x.^2+y.^2).^(5/2));
c12=-3*x.*y./(2*pi*(s2^2+x.^2+y.^2).^(5/2));
cbpy=c11-c12+i*(c21-c22);
cbpy=sqrt(E/2/sum(real(cbpy(:).*conj(cbpy(:)))))*cbpy; % ensure correct energy
